
/* Design an LCC type matching network */


#include <math.h>
#include <stdio.h>


double PI = 3.141592654;

struct complex	{
				double re;
				double im;
				};


/* Rectangular to polar conversion. This routine returns correct
	angles even in quadrants III and IV, where PI is added to the
	returned angle to perform the correction. */

void polar( val, mag, ang )
struct complex val;
double *mag, *ang;
{

/* The default return angle is zero - so 0 /_ 0 is returned for a
   rectangular input of 0 + 0j */

*ang = 0;

*mag = sqrt( val.re * val.re  +  val.im * val.im );

/* If number not purely imaginary, compute the angle. Otherwise, the angle is
   either + PI/2 or - PI/2  */

if (val.re != 0)	{
					*ang = atan( val.im / val.re);
					}
					else
						if (val.im > 0)	
										*ang = 1.570796327;
										else
										*ang = -1.570796327;

/* correct the angle if in quadrants III or IV */

if (val.re < 0)	
				*ang += PI;

}




/* Polar to rectangular conversion */

struct complex rect( mag, ang )
double mag, ang;
{
struct complex res;

res.re = mag * cos( ang );
res.im = mag * sin( ang );

return(res);
}


/* Complex number sum */

struct complex sum( v1, v2)
struct complex v1,v2;
{
struct complex v3;

v3.re = v1.re + v2.re;

v3.im = v1.im + v2.im;

return(v3);
}

/* Complex number product */

struct complex product( v1, v2)
struct complex v1,v2;
{
struct complex v3;

double v1_mag, v2_mag, v1_ang, v2_ang;
double res_mag, res_ang;

polar( v1, &v1_mag, &v1_ang);
polar( v2, &v2_mag, &v2_ang);

res_mag = v1_mag * v2_mag;

res_ang = v1_ang + v2_ang;

return( rect( res_mag, res_ang ) );
}


/* Complex number ratio  (computes v1/v2 )  */

struct complex ratio( v1, v2)
struct complex v1,v2;
{

double v1_mag, v2_mag, v1_ang, v2_ang;
double res_mag, res_ang;

polar( v1, &v1_mag, &v1_ang);
polar( v2, &v2_mag, &v2_ang);

return( rect( v1_mag / v2_mag ,  v1_ang - v2_ang)  );

}


/* Parallel element calculation -- product over sum */

struct complex parallel( v1, v2)
struct complex v1,v2;
{
struct complex result;

result = ratio( product(v1,v2) , sum(v1,v2) );

return( result );

}


/* Print complex number in rect notation */

void print( val )
struct complex val;
{

printf("%9.4lf  ", val.re );

if (val.im < 0)	printf("- %9.4lfj", -1*val.im);
				else
				printf("+ %9.4lfj", val.im);
}

/* Print complex number in polar notation */

void p_polar( val )
struct complex val;
{
double mag, ang;

polar( val, &mag, &ang);

printf("%9.4lf /_ %9.4lf ", mag, ang*(180/PI) );
}

/* ------------- Main program ----------------- */

void network_print()
{
printf("\n\n                 Ŀ          Ŀ\n");
printf("           Ĵ   L    Ĵ  C2    Ŀ\n");
printf("                             \n");
printf("                                                 \n");
printf("         Ŀ               Ŀ               Ŀ\n");
printf("                                                \n");
printf("         R1                C1                R2 \n");
printf("                                                \n");
printf("                                       \n");
printf("                                                  \n");
printf("                                        \n");
printf("                                            \n");
printf("                                                 \n");
}



void main()
{
int result;

printf("\nLCC Matching Network Design and Analysis\n\n");
printf("1) Design an LCC network\n2) Analyze existing network\n\nChoose?");

do	{
	result=csts();
	}
	while(result == 0);

printf("\n\n");

if (result == '1') lcc_design();

if (result == '2') lcc_eval();

}


/* Evalulate an LCC network */


void lcc_eval()
{
int result;
struct complex xc1,xc2,xl,r2,r1,rin;
double f,w,l,c1,c2;
char temp[5];

network_print();

printf("What operating frequency?");
scanf("%lf", &f);

w = 6.283185307 * f;


printf("Which termination is known? (1) for R1, (2) for R2?");

do	{
	result=csts();
	}
	while(result==0);

if ( (result < '1') || (result > '2')) exit(0);

printf("%c\nWhat is the value of L (Hy)?", result);
scanf("%lf", &l);

xl.re=0;
xl.im=w*l;

printf("What is the value of C1?");
scanf("%lf", &c1);
xc1.re=0;
xc1.im=-1/(w*c1);

printf("What is the value of C2?");
scanf("%lf", &c2);
xc2.re=0;
xc2.im=-1/(w*c2);


switch(result)	{
				case '1':
					printf("What is the REAL part of R1?");
					scanf("%lf", &r1.re);
					printf("What is the IMAG part of R1?");
					scanf("%lf", &r1.im);
					rin = sum( xc2, parallel( xc1, sum( xl, r1)));
					strcpy(temp,"R2");
					break;

				case '2':
					printf("What is the REAL part of R2?");
					scanf("%lf", &r2.re);
					printf("What is the IMAG part of R2?");
					scanf("%lf", &r2.im);
					rin = sum(xl, parallel(xc1, sum(xc2,r2)));
					strcpy(temp,"R1");
					break;

				}

printf("The input impedance at %s is: ", temp);
print( rin);
printf(" Ohms.\nPolar Zin = ");
p_polar( rin );
}



void lcc_design()
{
double r1,r2,q,f,w,xc1,xc2,c1,c2,xl,l,a,b;
int r1r2_swap;


r1r2_swap = 0;	/* If R2 > R1, this flag is zero. It denotes that R1 and
					R2 were swapped to perform the calculations. */


printf("LCC Matching Network Designer\n\n");

printf("What operating frequency?");
scanf("%lf", &f);

w = 6.283185307 * f;

printf("What network Q?");
scanf("%lf", &q);

printf("What is the input resistance?");
scanf("%lf", &r1);

printf("What is the output resistance?");
scanf("%lf", &r2);

if (r1 == r2)	{
				printf("Sorry, to use an LCC network the impedances cannot be equal!\n");
				exit(0);
				}


if (r2 < r1)	{
				f=r2;
				r2=r1;
				r1=f;
				r1r2_swap = TRUE;
				}


a = sqrt( (r1 * (q*q + 1) / r2 ) - 1 );

b = r1 * (q*q + 1);

xc1 = b/(q-a);
xc2 = a*r2;
xl  = q*r1;


network_print();

if (r1r2_swap== FALSE )
			{
			printf("R1 = INPUT  = %9.4lf Ohms\n", r1);
			printf("R2 = OUTPUT = %9.4lf Ohms\n\n", r2);
			}
			else
			{
			printf("R1 = OUTPUT = %9.4lf Ohms\n", r1);
			printf("R2 = INPUT  = %9.4lf Ohms\n\n", r2);
			}

printf("L = %9.4lf uH, C1 = %9.4lf pF, C2 = %9.4lf pF\n\n",
		1e6*xl/w, 1e12*1/(xc1*w), 1e12*1/(xc2*w) );

printf("XL = %9.4lf, XC1 = %9.4lf, XC2 = %9.4lf OHMS\n\n",
		xl, xc1, xc2);

printf("Q = %5.1lf, F0 = %9.2lf KHz\n" , q , f / 1e3 );

}


