
/* These are fundamental subroutines for dealing with complex arithmetic.
	Product, sum, ratio, and parallel operations are supported. All
	operators use rectangular form; conversion between rectangular and
	polar notation is supported. */


#include <stdio.h>
#include <math.h>

struct complex	{
				double re;
				double im;
				};


/* Rectangular to polar conversion */

void polar( val, mag, ang )
struct complex val;
double *mag, *ang;
{

*mag = sqrt( val.re * val.re  +  val.im * val.im );

if (val.re != 0)	{
					*ang = atan( val.im / val.re);
					}
					else
						if (val.im > 0)	
										*ang = 1.5707963;
										else
										*ang = -1.5707963;

}


/* Polar to rectangular conversion */

struct complex rect( mag, ang )
double mag, ang;
{
struct complex res;

res.re = mag * cos( ang );
res.im = mag * sin( ang );

return(res);
}


/* Complex number sum */

struct complex sum( v1, v2)
struct complex v1,v2;
{
struct complex v3;

v3.re = v1.re + v2.re;

v3.im = v1.im + v2.im;

return(v3);
}

/* Complex number product */

struct complex product( v1, v2)
struct complex v1,v2;
{
struct complex v3;

double v1_mag, v2_mag, v1_ang, v2_ang;
double res_mag, res_ang;

polar( v1, &v1_mag, &v1_ang);
polar( v2, &v2_mag, &v2_ang);

res_mag = v1_mag * v2_mag;

res_ang = v1_ang + v2_ang;

return( rect( res_mag, res_ang ) );
}


/* Complex number ratio  (computes v1/v2 )  */

struct complex ratio( v1, v2)
struct complex v1,v2;
{

double v1_mag, v2_mag, v1_ang, v2_ang;
double res_mag, res_ang;

polar( v1, &v1_mag, &v1_ang);
polar( v2, &v2_mag, &v2_ang);

return( rect( v1_mag / v2_mag ,  v1_ang - v2_ang)  );

}


/* Parallel element calculation -- product over sum */

struct complex parallel( v1, v2)
struct complex v1,v2;
{
struct complex result;

result = ratio( product(v1, v2), sum( v1, v2) );

return(result);

}


/* Print complex number in rect notation */

void print( val )
struct complex val;
{

printf("%9.4lf  +  %9.4lfj", val.re, val.im);
}



/*  ------------------ main program ------------------ */


void main()
{
struct complex z1,z2;
struct complex result;

/* This is a test of the package */

printf("Enter Z1 real part:");
scanf("%lf", &z1.re);

printf("Enter Z1 imag part:");
scanf("%lf", &z1.im);

printf("Enter z2 real part:");
scanf("%lf", &z2.re);

printf("Enter z2 imag part:");
scanf("%lf", &z2.im);

result = parallel(z1, z2);

printf("The parallel form is ");
print( result );

}
