
/* PI network input impedance calculation.

   Author: Wheeler, T.

   Used in EET-363 Electronic Communications	*/


#include <stdio.h>
#include <math.h>



double PI = 3.141592654;

struct cmplx	{
		double re;
		double im;
		};


/* Rectangular to polar conversion. This routine returns correct
	angles even in quadrants III and IV, where PI is added to the
	returned angle to perform the correction. */

void polar( val, mag, ang )
struct cmplx val;
double *mag, *ang;
{

/* The default return angle is zero - so 0 /_ 0 is returned for a
   rectangular input of 0 + 0j */

*ang = 0;

*mag = sqrt( val.re * val.re  +  val.im * val.im );

/* If number not purely imaginary, compute the angle. Otherwise, the angle is
   either + PI/2 or - PI/2  */

if (val.re != 0)	{
					*ang = atan( val.im / val.re);
					}
					else
						if (val.im > 0)	
										*ang = 1.570796327;
										else
										*ang = -1.570796327;

/* correct the angle if in quadrants III or IV */

if (val.re < 0)	
				*ang += PI;

}




/* Polar to rectangular conversion */

struct cmplx rect( mag, ang )
double mag, ang;
{
struct cmplx res;

res.re = mag * cos( ang );
res.im = mag * sin( ang );

return(res);
}


/* Complex number sum */

struct cmplx sum( v1, v2)
struct cmplx v1,v2;
{
struct cmplx v3;

v3.re = v1.re + v2.re;

v3.im = v1.im + v2.im;

return(v3);
}

/* Complex number product */

struct cmplx product( v1, v2)
struct cmplx v1,v2;
{
struct cmplx v3;

double v1_mag, v2_mag, v1_ang, v2_ang;
double res_mag, res_ang;

polar( v1, &v1_mag, &v1_ang);
polar( v2, &v2_mag, &v2_ang);

res_mag = v1_mag * v2_mag;

res_ang = v1_ang + v2_ang;

return( rect( res_mag, res_ang ) );
}


/* Complex number ratio  (computes v1/v2 )  */

struct cmplx ratio( v1, v2)
struct cmplx v1,v2;
{

double v1_mag, v2_mag, v1_ang, v2_ang;
double res_mag, res_ang;

polar( v1, &v1_mag, &v1_ang);
polar( v2, &v2_mag, &v2_ang);

return( rect( v1_mag / v2_mag ,  v1_ang - v2_ang)  );

}


/* Parallel element calculation -- product over sum */

struct cmplx parallel( v1, v2)
struct cmplx v1,v2;
{
struct cmplx result;

result = ratio( product(v1,v2) , sum(v1,v2) );

return( result );

}


/* Print complex number in rect notation */

void print( val )
struct cmplx val;
{

printf("%9.4lf  ", val.re );

if (val.im < 0)	printf("- %9.4lfj", -1*val.im);
				else
				printf("+ %9.4lfj", val.im);
}

/* Print complex number in polar notation */

void p_polar( val )
struct cmplx val;
{
double mag, ang;

polar( val, &mag, &ang);

printf("%9.4lf /_ %9.4lf ", mag, ang*(180/PI) );
}



/*  ------------------ main program ------------------ */

void plot_response()
{
double freq,ndx,w;
double c1,c2,l;
double mag, ang;

struct cmplx xc1,xc2,xl,rl,i1,zin;
struct cmplx v1,v2,gain;

/* The network will be driven by a unit current source of 1 /_ 0 */

i1.re = 1;
i1.im = 0;

printf("PI-Network frequency response. The left side of the network will\n");
printf("be driven by a unit current source of 1 /_ 0 for the gain and\n");
printf("phase analysis. Frequency range will be .2f0 to 2.1 f0\n\n");

printf("What is the operating frequency (Hz)?");
scanf("%lf", &freq);

printf("\nWhat is the value of C1 (input capacitor, left side)?");
scanf("%lf", &c1);

printf("\nWhat is the value of C2 (output capacitor, right side)?");
scanf("%lf", &c2);

printf("\nWhat is the value of L, the inductor?");
scanf("%lf", &l);

printf("\nWhat is the REAL part of the load resistance ZL?");
scanf("%lf", &rl.re);

printf("\nWhat is the IMAGINARY part of ZL (0 if none) ?");
scanf("%lf", &rl.im);

/* Calculate results. First convert frequency to r/s, then find all
   circuit impedances. Then evaluate. */

freq = freq * 2 * PI;	/* 2_PI *f */

/* Evaluation loop */

printf("w/w0    Av(dB)   Phase Angle\n");
printf("----------------------------\n");

for( ndx = 0.2; ndx < 2.2; ndx += 0.2)
	{
	w = freq * ndx;

	xl.re = 0;
	xl.im = w * l;

	xc1.re = 0;
	xc1.im = - 1 / (w * c1);

	xc2.re = 0;
	xc2.im = -1 / (w * c2);

   zin = parallel( xc1, sum(xl, parallel(xc2,rl)));

   v1 = product( zin, i1);



	v2 = product( ratio( parallel( rl, xc2),
			sum(xl, parallel(rl, xc2))) , v1);

   gain = ratio (v2, v1);

	polar( gain, &mag, &ang);

	printf("%9.1lf%9.4lf%9.4lf\n", ndx, 20*log10( mag ), 180/PI*ang );
	}

}


void analyze_pi()
{
double freq;
double c1,c2,l;
double mag, ang;

struct cmplx xc1,xc2,xl,rl;
struct cmplx zi;
struct cmplx result;



printf("Input impedance of a PI network.\n\n");

printf("What is the operating frequency (Hz)?");
scanf("%lf", &freq);

printf("\nWhat is the value of C1 (input capacitor, left side)?");
scanf("%lf", &c1);

printf("\nWhat is the value of C2 (output capacitor, right side)?");
scanf("%lf", &c2);

printf("\nWhat is the value of L, the inductor?");
scanf("%lf", &l);

printf("\nWhat is the REAL part of the load resistance ZL?");
scanf("%lf", &rl.re);

printf("\nWhat is the IMAGINARY part of ZL (0 if none) ?");
scanf("%lf", &rl.im);

/* Calculate results. First convert frequency to r/s, then find all
   circuit impedances. Then evaluate. */

freq = freq * 2 * PI;

xl.re = 0;
xl.im = freq * l;

xc1.re = 0;
xc1.im = - 1 / (freq * c1);

xc2.re = 0;
xc2.im = -1 / (freq * c2);


result = parallel( xc1, sum(xl, parallel(xc2,rl)));

printf("The input impedance is ");
print( result );
printf(" Ohms.\n");

polar( result, &mag, &ang);

printf("Polar form input impedance:   %9.4lf /_ %9.4lf  Ohms\n", mag, ang * 57.29578 );



}


/* Designs a PI network */


void pides()
{
double r1,r2,q1,q2,ri,c1,c2,l,f,w;

printf("\nPI network designer. C1 is the input cap and C2 is the output cap.\n\n");

printf("R1 (input impedance) value?");
scanf("%lf", &r1);

printf("R2 (output impedance) value?");
scanf("%lf", &r2);

printf("What Q for LH side of network?");
scanf("%lf", &q1);

printf("What center frequency?");
scanf("%lf", &f);

w = 2 * PI * f;

ri = r1 / (q1*q1 + 1);

q2 = sqrt( r2/ri -1 );

c1 = q1 / ( w * r1 );

c2 = q2 / ( w * r2 );

l = ri * ( q1 + q2 ) / w ;

printf("\nC1 = %9.5lf pF", c1 * 1e12 );

printf("\nC2 = %9.5lf pF", c2 * 1e12 );

printf("\nL  = %9.5lf uH",  l * 1e6 );

printf("\n\n ----- Network Parameters ------ \n\n");

printf("Q1 = %lf, Q2 = %lf, Ri = %lf\n\n", q1,q2,ri);

}


void main()
{
int result;

printf("\nPI Network Design and Analysis\n\n");
printf("1) Design a PI network\n2) Analyze Zin of existing network\n");
printf("3) Plot frequency response of existing network\n\nChoose?");

do	{
	result=getche();
	}
	while(result == 0);

printf("\n\n");

if (result == '1') pides();

if (result == '2') analyze_pi();

if (result == '3') plot_response();

}


