/* This is a simple small-signal, class A RF design program.
   It uses the R-parameter system and assumes ideal components throughout.
   Distributed capacitance, and inductance are ignored, as well as
   device inter-electrode capacitance. 
   Used in EET-363 ELECTRONIC COMMUNICATIONS, DeVry Institute - KC

   V1.1 July 1995

   (c) 1995 Tom A. Wheeler

			*/


#include <stdio.h>
#include <math.h>


double beta = 100.;		/* Program assumes beta DC to be 100 */
double p2 = 6.2831853;	/* constant: 2PI */

void main()
{
double vcc, po, ie, ve, vb, re, rl, x, l, c, rb1, rb2, av, q, bw, f;
double r_prime, ac_re, r_prime_l, ns_np;
char response;

printf("\nWelcome to the small-signal class A designer.\n\nWhat is Vcc?");
scanf("%lf", &vcc);

printf("What power output, in mW, is needed?");
scanf("%lf", &po);
po=po/1e3;

printf("Do you know the value for RL (Y/N)?");
response = toupper( getchar() );

r_prime_l = .81*vcc*vcc/(2*po);

if (response == 'Y')
	{
	printf("\n\nWhat is the load resistance?");
	scanf("%lf", &rl);
	}
	else
	{
	rl = r_prime_l;
	printf("\n\nRL has been chosen as %lf Ohms.\n\n", rl);
	}


printf("What is the passband voltage gain (Av) ?");
scanf("%lf", &av);

printf("What is the operating frequency (F0) ?");
scanf("%lf", &f);

printf("What is the required bandwidth (BW)?");
scanf("%lf", &bw);

q = f / bw;
if (q < 10)
   printf("\nWARNING: Q < 10 (Q = %.1lf), BW is only approximate!\n\n",q);




/* DC calculations */

ie = 2*po/(.9 * vcc );		/* Compute DC emitter current */

ve = vcc / 10;				/* DC emitter voltage */
re = ve / ie;				/* Recommended Emitter Resistance (RE1+RE2) */

vb = ve + .7;
rb2 = vb / ( ie / (beta /10.) );	/* Upper and lower base resistors */
rb1 = (vcc - vb) / ( ie / (beta / 10.) );

/* Do the calculations for AC */

x = r_prime_l / q;					/* Compute Z0, the resonant reactance */

l = x / (f * p2);			/* Find L and C values. */
c = 1 / (f * p2 * x);

r_prime = .025 / ie;


/* If RL was NOT known, then RL was computed by the program and the 
load will be directly coupled to the amplifier tank. Otherwise, the program
must use an output transformer and adjust parameters for that. */


if (response != 'Y')
	{
	ac_re = ( rl / av) - r_prime;
	}

	else

	{
	ns_np = sqrt(rl / r_prime_l );
	ac_re = ( r_prime_l * ns_np ) / av - r_prime;
	}

printf("\n\n----------------- DC PARAMETERS -------------------------\n\n");

printf("Ie = %lf mA,  Ve = %lf Volts,  Vb = %lf Volts\n",
		ie*1000., ve, vb);
printf("Rbase 1 (top) = %lf, Rbase 2 (bottom) = %lf\nDC RE = %lf Ohms\n\n",
			  rb1, rb2, re);

printf("Transistor Power Dissipation: %lf Watts\n\n", .9*vcc*ie );

printf("----------------- AC PARAMETERS -------------------------\n\n");
printf("r'e = %lf Ohms,    AC rE = %lf Ohms\n\n", r_prime, ac_re);
printf("L = %lf uH, C = %lf uF\n\n", l * 1e6, c * 1e6);
printf("Q = %.2lf\n\n", q);
if (rl == r_prime_l)
	{
	printf("\nThe load is directly coupled to the tank. RL = %lf Ohms.\n",rl);
	}
	else
	{
	printf("\nCoupling transformer used. Np:Ns = %lf\n", 1 / ns_np);
	printf("R'L = %lf Ohms\n", r_prime_l );
	}

}
